@@ -1,5 +1,5 @@
 # mako/template.py
-# Copyright (C) 2006-2011 the Mako authors and contributors <see AUTHORS file>
+# Copyright (C) 2006-2012 the Mako authors and contributors <see AUTHORS file>
 #
 # This module is part of Mako and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -8,7 +8,7 @@
 template strings, as well as template runtime operations."""
 
 from mako.lexer import Lexer
-from mako import runtime, util, exceptions, codegen
+from mako import runtime, util, exceptions, codegen, cache
 import imp, os, re, shutil, stat, sys, tempfile, time, types, weakref
 
  
@@ -47,18 +47,24 @@ class Template(object):
      through to the buffer.   New in 0.4 to provide the same
      behavior as that of the previous series.  This flag is forced
      to True if disable_unicode is also configured.
-
-    :param cache_dir: Filesystem directory where cache files will be
-     placed.  See :ref:`caching_toplevel`.
+    
+    :param cache_args: Dictionary of cache configuration arguments that 
+     will be passed to the :class:`.CacheImpl`.   See :ref:`caching_toplevel`.
+     
+    :param cache_dir: Deprecated; use the 'dir' argument in the
+     cache_args dictionary.  See :ref:`caching_toplevel`.
 
     :param cache_enabled: Boolean flag which enables caching of this
      template.  See :ref:`caching_toplevel`.
 
-    :param cache_type: Type of Beaker caching to be applied to the 
-     template. See :ref:`caching_toplevel`.
+    :param cache_impl: String name of a :class:`.CacheImpl` caching
+     implementation to use.   Defaults to 'beaker'.
+
+    :param cache_type: Deprecated; use the 'type' argument in the 
+     cache_args dictionary.  See :ref:`caching_toplevel`.
  
-    :param cache_url: URL of a memcached server with which to use
-     for caching.  See :ref:`caching_toplevel`.
+    :param cache_url: Deprecated; use the 'URL' argument in the
+     cache_args dictionary.  See :ref:`caching_toplevel`.
 
     :param default_filters: List of string filter names that will
      be applied to all expressions.  See :ref:`filtering_default_filters`.
@@ -104,6 +110,39 @@ class Template(object):
     :param module_filename: Overrides the filename of the generated 
      Python module file. For advanced usage only.
  
+    :param module_writer: A callable which overrides how the Python
+     module is written entirely.  The callable is passed the 
+     encoded source content of the module and the destination
+     path to be written to.   The default behavior of module writing
+     uses a tempfile in conjunction with a file move in order
+     to make the operation atomic.   So a user-defined module
+     writing function that mimics the default behavior would be::
+
+        import tempfile
+        import os
+        import shutil
+
+        def module_writer(source, outputpath):
+            (dest, name) = \\
+                tempfile.mkstemp(
+                    dir=os.path.dirname(outputpath)
+                )
+ 
+            os.write(dest, source)
+            os.close(dest)
+            shutil.move(name, outputpath)
+
+        from mako.template import Template
+        mytemplate = Template(
+                        file="index.html", 
+                        module_directory="/path/to/modules", 
+                        module_writer=module_writer
+                    )
+
+     The function is provided for unusual configurations where
+     certain platform-specific permissions or other special
+     steps are needed.
+
     :param output_encoding: The encoding to use when :meth:`.render` 
      is called.  
      See :ref:`usage_unicode` as well as :ref:`unicode_toplevel`.
@@ -139,19 +178,22 @@ class Template(object):
                     output_encoding=None, 
                     encoding_errors='strict', 
                     module_directory=None, 
+                    cache_args=None,
+                    cache_impl='beaker',
+                    cache_enabled=True,
                     cache_type=None, 
                     cache_dir=None, 
                     cache_url=None, 
                     module_filename=None, 
                     input_encoding=None, 
                     disable_unicode=False,
+                    module_writer=None,
                     bytestring_passthrough=False, 
                     default_filters=None, 
                     buffer_filters=(), 
                     strict_undefined=False,
                     imports=None, 
-                    preprocessor=None, 
-                    cache_enabled=True):
+                    preprocessor=None):
         if uri:
             self.module_id = re.sub(r'\W', "_", uri)
             self.uri = uri
@@ -180,6 +222,7 @@ class Template(object):
         self.disable_unicode = disable_unicode
         self.bytestring_passthrough = bytestring_passthrough or disable_unicode
         self.strict_undefined = strict_undefined
+        self.module_writer = module_writer
 
         if util.py3k and disable_unicode:
             raise exceptions.UnsupportedError(
@@ -232,11 +275,32 @@ class Template(object):
         self.format_exceptions = format_exceptions
         self.error_handler = error_handler
         self.lookup = lookup
-        self.cache_type = cache_type
-        self.cache_dir = cache_dir
-        self.cache_url = cache_url
+
+        self.module_directory = module_directory
+
+        self._setup_cache_args(
+            cache_impl, cache_enabled, cache_args,
+            cache_type, cache_dir, cache_url
+        )
+
+    def _setup_cache_args(self, 
+                cache_impl, cache_enabled, cache_args,
+                cache_type, cache_dir, cache_url):
+        self.cache_impl = cache_impl
         self.cache_enabled = cache_enabled
- 
+        if cache_args:
+            self.cache_args = cache_args
+        else:
+            self.cache_args = {}
+
+        # transfer deprecated cache_* args
+        if cache_type:
+            self.cache_args['type'] = cache_type
+        if cache_dir:
+            self.cache_args['dir'] = cache_dir
+        if cache_url:
+            self.cache_args['url'] = cache_url
+
     def _compile_from_file(self, path, filename):
         if path is not None:
             util.verify_directory(os.path.dirname(path))
@@ -247,7 +311,8 @@ class Template(object):
                             self, 
                             open(filename, 'rb').read(), 
                             filename, 
-                            path)
+                            path,
+                            self.module_writer)
             module = imp.load_source(self.module_id, path, open(path, 'rb'))
             del sys.modules[self.module_id]
             if module._magic_number != codegen.MAGIC_NUMBER:
@@ -255,7 +320,8 @@ class Template(object):
                             self, 
                             open(filename, 'rb').read(), 
                             filename, 
-                            path)
+                            path,
+                            self.module_writer)
                 module = imp.load_source(self.module_id, path, open(path, 'rb'))
                 del sys.modules[self.module_id]
             ModuleInfo(module, path, self, filename, None, None)
@@ -283,9 +349,19 @@ class Template(object):
  
         return _get_module_info_from_callable(self.callable_).code
  
-    @property
+    @util.memoized_property
     def cache(self):
-        return self.module._template_cache
+        return cache.Cache(self)
+
+    @property
+    def cache_dir(self):
+        return self.cache_args['dir']
+    @property
+    def cache_url(self):
+        return self.cache_args['url']
+    @property
+    def cache_type(self):
+        return self.cache_args['type']
  
     def render(self, *args, **data):
         """Render the output of this template as a string.
@@ -369,10 +445,12 @@ class ModuleTemplate(Template):
                         format_exceptions=False,
                         error_handler=None, 
                         lookup=None, 
-                        cache_type=None,
+                        cache_args=None,
+                        cache_impl='beaker',
+                        cache_enabled=True,
+                        cache_type=None, 
                         cache_dir=None, 
                         cache_url=None, 
-                        cache_enabled=True
     ):
         self.module_id = re.sub(r'\W', "_", module._template_uri)
         self.uri = module._template_uri
@@ -404,10 +482,10 @@ class ModuleTemplate(Template):
         self.format_exceptions = format_exceptions
         self.error_handler = error_handler
         self.lookup = lookup
-        self.cache_type = cache_type
-        self.cache_dir = cache_dir
-        self.cache_url = cache_url
-        self.cache_enabled = cache_enabled
+        self._setup_cache_args(
+            cache_impl, cache_enabled, cache_args,
+            cache_type, cache_dir, cache_url
+        )
  
 class DefTemplate(Template):
     """a Template which represents a callable def in a parent
@@ -502,7 +580,7 @@ def _compile_text(template, text, filename):
     exec code in module.__dict__, module.__dict__
     return (source, module)
 
-def _compile_module_file(template, text, filename, outputpath):
+def _compile_module_file(template, text, filename, outputpath, module_writer):
     identifier = template.module_id
     lexer = Lexer(text, 
                     filename, 
@@ -522,17 +600,20 @@ def _compile_module_file(template, text, filename, outputpath):
                                 disable_unicode=template.disable_unicode,
                                 strict_undefined=template.strict_undefined)
  
-    # make tempfiles in the same location as the ultimate 
-    # location.   this ensures they're on the same filesystem,
-    # avoiding synchronization issues.
-    (dest, name) = tempfile.mkstemp(dir=os.path.dirname(outputpath))
- 
     if isinstance(source, unicode):
         source = source.encode(lexer.encoding or 'ascii')
+
+    if module_writer:
+        module_writer(source, outputpath)
+    else:
+        # make tempfiles in the same location as the ultimate 
+        # location.   this ensures they're on the same filesystem,
+        # avoiding synchronization issues.
+        (dest, name) = tempfile.mkstemp(dir=os.path.dirname(outputpath))
  
-    os.write(dest, source)
-    os.close(dest)
-    shutil.move(name, outputpath)
+        os.write(dest, source)
+        os.close(dest)
+        shutil.move(name, outputpath)
 
 def _get_module_info_from_callable(callable_):
     return _get_module_info(callable_.func_globals['__name__'])
